# coding: utf-8

"""
    PKS

    PKS API  # noqa: E501

    OpenAPI spec version: 1.1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class Usage(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'owner': 'str',
        'totals': 'UsageTotals',
        'clusters': 'list[ClusterUsage]'
    }

    attribute_map = {
        'owner': 'owner',
        'totals': 'totals',
        'clusters': 'clusters'
    }

    def __init__(self, owner=None, totals=None, clusters=None):  # noqa: E501
        """Usage - a model defined in Swagger"""  # noqa: E501

        self._owner = None
        self._totals = None
        self._clusters = None
        self.discriminator = None

        self.owner = owner
        self.totals = totals
        self.clusters = clusters

    @property
    def owner(self):
        """Gets the owner of this Usage.  # noqa: E501


        :return: The owner of this Usage.  # noqa: E501
        :rtype: str
        """
        return self._owner

    @owner.setter
    def owner(self, owner):
        """Sets the owner of this Usage.


        :param owner: The owner of this Usage.  # noqa: E501
        :type: str
        """
        if owner is None:
            raise ValueError("Invalid value for `owner`, must not be `None`")  # noqa: E501

        self._owner = owner

    @property
    def totals(self):
        """Gets the totals of this Usage.  # noqa: E501


        :return: The totals of this Usage.  # noqa: E501
        :rtype: UsageTotals
        """
        return self._totals

    @totals.setter
    def totals(self, totals):
        """Sets the totals of this Usage.


        :param totals: The totals of this Usage.  # noqa: E501
        :type: UsageTotals
        """
        if totals is None:
            raise ValueError("Invalid value for `totals`, must not be `None`")  # noqa: E501

        self._totals = totals

    @property
    def clusters(self):
        """Gets the clusters of this Usage.  # noqa: E501


        :return: The clusters of this Usage.  # noqa: E501
        :rtype: list[ClusterUsage]
        """
        return self._clusters

    @clusters.setter
    def clusters(self, clusters):
        """Sets the clusters of this Usage.


        :param clusters: The clusters of this Usage.  # noqa: E501
        :type: list[ClusterUsage]
        """
        if clusters is None:
            raise ValueError("Invalid value for `clusters`, must not be `None`")  # noqa: E501

        self._clusters = clusters

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Usage, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Usage):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
